//######################################################################
// Micha Kurzela @ 2015
// Zegar binarny
// ATmega48V-10AU, taktowanie kwarcem 4MHz + wyczony CKDIV8
//######################################################################

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#define SW_M bit_is_clear(PINB,PINB3) // ustawianie minut
#define SW_H bit_is_clear(PINB,PINB4) //ustawianie godzin
#define SUPPLY bit_is_clear(PINC,PINC1) //sprawdzanie, czy mamy zasilanie, TRUE jeli tak

//######################################################################
//######################################################################
//######################################################################

//deklaracja zm ienych globalnych
volatile uint8_t godziny = 0; 
volatile uint8_t minuty = 0; 
volatile uint8_t sekundy = 0; //zmienne "czasowe"
volatile uint8_t wiersz = 0; //aktualnie uruchomiony wiersz
//0 - sekundy, 1 - minuty, 2 - godziny
volatile uint8_t licznik = 0; //zmienna liczca przerwania od Timer0 - zlicza ich 250
volatile uint8_t supply = 0; //0 - jest zasilanie, 1 - brak zasilania

//######################################################################
//######################################################################
//######################################################################

void podziel_liczbe(uint8_t liczba){ //najpierw zaczamy diody, a potem cay wiersz
PORTD &= 0b11111000; //wyczenie wiersza, aby nie miga podczas ustawiania diod

//porwnywanie z wagami bitw i ewentualne ich odejmowanie, a do zera
if(liczba >= 32){ PORTB |= 0b00000100; liczba -= 32;} else { PORTB &= 0b11111011;}
if(liczba >= 16){ PORTB |= 0b00000010; liczba -= 16;} else { PORTB &= 0b11111101;}
if(liczba >= 8){ PORTB |= 0b00000001; liczba -= 8;} else { PORTB &= 0b11111110;}
if(liczba >= 4){ PORTD |= 0b10000000; liczba -= 4;} else { PORTD &= 0b01111111;}
if(liczba >= 2){ PORTD |= 0b01000000; liczba -= 2;} else { PORTD &= 0b10111111;}
if(liczba >= 1){ PORTD |= 0b00100000; liczba -= 1;} else { PORTD &= 0b11011111;}

//zostao to wykonane instrukcjami IF aby byo bardziej przejrzycie
if(wiersz == 0){PORTD |= 0b00000001; } //zaczony wiersz sekund
if(wiersz == 1){PORTD |= 0b00000010; } //zaczony wiersz minut
if(wiersz == 2){PORTD |= 0b00000100; } //zaczony wiersz godzin
}

//######################################################################
//######################################################################
//######################################################################

ISR(TIMER0_OVF_vect){
TCNT0 = 6; //odliczanie 250
licznik++;
if(licznik >= 250){licznik = 0; sekundy++;}
if(sekundy >= 60){sekundy = 0; minuty++;}
if(minuty >= 60){minuty = 0; godziny++;}
if(godziny >= 24){godziny = 0;}

if(supply == 0){ //kiedy jest zasilanie, to wywietlaj
wiersz++; //inkrementacja w kadym przerwaniu
if(wiersz >= 3){ wiersz = 0;}

if(wiersz == 0){ podziel_liczbe(sekundy); } //wywietlenie jednej wartoci
if(wiersz == 1){ podziel_liczbe(minuty); }
if(wiersz == 2){ podziel_liczbe(godziny); }}

if(supply == 1){ //kiedy nie ma zasilania
PORTD &= 0;  //wycz wiersze i kolumny
PORTB &= 0b00011000;
}}

//######################################################################
//######################################################################
//######################################################################

int main (void){
//sprawdzanie obecnoci zasilania
//aby wywietlanie w przerwaniu si przypadkowo nie uaktywnio
if(SUPPLY){supply = 0;} else {supply = 1;} 

DDRB = 0b00000111; //konfiguracja we/wy
DDRC = 0b11111101;
DDRD = 0b11111111;

PORTB = 0b00011000;
PORTC = 255;
PORTD = 0;

TCCR0A = 0;
TCCR0B = 0b00000011; //przeskaler przez 64
TCNT0 = 6; //250 taktw dla czstotliwo przerwa 250Hz
TIMSK0 = 1; //zezwolenie na przerwania od przepenienia
TIFR0 = 1;
sei(); //aktywacja systemu przerwa


while(1){

if(SUPPLY){supply = 0;} else {supply = 1;} //sprawdzanie obecnoci zasilania

if(supply == 0){ //nastawy czasu tylko przy zasilaniu
if(SW_M){minuty++; sekundy = 0; licznik = 0; if(minuty >= 60){minuty = 0;} _delay_ms(120);}
if(SW_H){godziny++; sekundy = 0; licznik = 0; if(godziny >= 24){godziny = 0;} _delay_ms(300);}
}}}

